%--------------------------------------------------------------------------
function Photo=MatBayerDecode(CFA)
%--------------------------------------------------------------------------
load('rawdecode.mat','FileName','UniqueCamera','RawInfo');
load('rawcamera.mat','Camera');
load('rawconfig.mat','DataPath','BayerCfaSave',...
    'BayerInterpolate','MatlabBayerInterp',...
    'Histogram','HistCentre','HistRange');
CFA=double(CFA);
if isfield(Camera,'CFARepeatPatternDimension') && ...
        isfield(Camera,'CFAPatternValue')
%--------------------------------------------------------------------------
% CFA Repeat Pattern Dimension: Horizontal repeat pixel unit: 2,
%      Vertical repeat pixel unit: 2
% CFA Pattern 2: [Red, Green], [Green, Blue]
%      0 - red, 1 - green, 2 - blue
%--------------------------------------------------------------------------
    SrcWidth=Camera.DefaultCropSize(1);
    SrcHeight=Camera.DefaultCropSize(2);
    PhotoWidth=SrcWidth/2;
    PhotoHeight=SrcHeight/2;

    row=Camera.ActiveArea(1)+Camera.DefaultCropOrigin(2);
    col=Camera.ActiveArea(2)+Camera.DefaultCropOrigin(1);
    CFA=CFA(row+1:row+SrcHeight,col+1:col+SrcWidth);
    mapstr='rgb';
    CFAPattern=reshape(Camera.CFAPatternValue',...
        Camera.CFARepeatPatternDimension(1),...
        Camera.CFARepeatPatternDimension(2))';
    BayerStructOrigin=reshape(mapstr(CFAPattern+1)',1,4);
%--------------------------------------------------------------------------
% remainder nonzero, row or is odd, BayerPattern must be changed
%--------------------------------------------------------------------------
    if mod(Camera.DefaultCropOrigin(2),2)~=0
        t=CFAPattern(2,:);
        CFAPattern(2,:)=CFAPattern(1,:);
        CFAPattern(1,:)=t;
    end
    if mod(Camera.DefaultCropOrigin(1),2)~=0
        t=CFAPattern(:,2);
        CFAPattern(:,2)=CFAPattern(:,1);
        CFAPattern(:,1)=t;
    end
    BayerStruct=reshape(mapstr(CFAPattern+1)',1,4);
%--------------------------------------------------------------------------
    if Camera.BlackLevelRepeatDim(1)==2 && ...
            Camera.BlackLevelRepeatDim(2)==2
        BlackLevelOrigin(1:4)=Camera.BlackLevel(1:4);
    elseif Camera.BlackLevelRepeatDim(1)==1 && ...
            Camera.BlackLevelRepeatDim(2)==1;
        BlackLevelOrigin(1:4)=Camera.BlackLevel;
    else
        Disp('Unknown - BlackLevel');
    end
%--------------------------------------------------------------------------
% Get RGGB image data
%--------------------------------------------------------------------------
    BayerColorData=zeros(PhotoHeight,PhotoWidth,4);
    switch BayerStruct
        case 'rggb'         % 1-R, 2-G1, 3-G2, 4-B
            BayerColorData(1:PhotoHeight,1:PhotoWidth,1)=...
                CFA(1:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,2)=...
                CFA(1:2:SrcHeight,2:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,3)=...
                CFA(2:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,4)=...
                CFA(2:2:SrcHeight,2:2:SrcWidth);
            BlackLevel(1)=BlackLevelOrigin(1);
            BlackLevel(2)=BlackLevelOrigin(2);
            BlackLevel(3)=BlackLevelOrigin(3);
            BlackLevel(4)=BlackLevelOrigin(4);
        case 'bggr'         % 1-B, 2-G1, 3-G2, 4-R
            BayerColorData(1:PhotoHeight,1:PhotoWidth,4)=...
                CFA(1:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,2)=...
                CFA(1:2:SrcHeight,2:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,3)=...
                CFA(2:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,1)=...
                CFA(2:2:SrcHeight,2:2:SrcWidth);
            BlackLevel(4)=BlackLevelOrigin(1);
            BlackLevel(2)=BlackLevelOrigin(2);
            BlackLevel(3)=BlackLevelOrigin(3);
            BlackLevel(1)=BlackLevelOrigin(4);
        case 'grbg'         % 1-G1, 2-R, 3-B, 4-G2
            BayerColorData(1:PhotoHeight,1:PhotoWidth,2)=...
                CFA(1:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,1)=...
                CFA(1:2:SrcHeight,2:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,4)=...
                CFA(2:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,3)=...
                CFA(2:2:SrcHeight,2:2:SrcWidth);
            BlackLevel(2)=BlackLevelOrigin(1);
            BlackLevel(1)=BlackLevelOrigin(2);
            BlackLevel(4)=BlackLevelOrigin(3);
            BlackLevel(3)=BlackLevelOrigin(4);
        case 'gbrg'       % 1-G1, 2-B, 3-R, 4-G2
            BayerColorData(1:PhotoHeight,1:PhotoWidth,2)=...
                CFA(1:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,4)=...
                CFA(1:2:SrcHeight,2:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,1)=...
                CFA(2:2:SrcHeight,1:2:SrcWidth);
            BayerColorData(1:PhotoHeight,1:PhotoWidth,3)=...
                CFA(2:2:SrcHeight,2:2:SrcWidth);
            BlackLevel(2)=BlackLevelOrigin(1);            
            BlackLevel(4)=BlackLevelOrigin(2);
            BlackLevel(1)=BlackLevelOrigin(3);
            BlackLevel(3)=BlackLevelOrigin(4);
        otherwise
            disp('-------- Error BayerStruct ---------');
            INTERRUPT
    end
    WhiteLevelOrigin=Camera.WhiteLevel;
    WhiteLevel=Camera.WhiteLevel;
%--------------------------------------------------------------------------
    if Histogram~=0
        save('rawdecode.mat','BlackLevel','WhiteLevel','-append');
        MatImageHistRaw(BayerColorData,HistCentre,HistRange);
    end
%--------------------------------------------------------------------------
    Data=MatRescaling(BayerColorData,BlackLevel,WhiteLevel);
    if BayerInterpolate~=0 && MatlabBayerInterp==0
        disp('-------- rivershaw Bayer Interpolation --------');
        Photo=MatBayer2Photo(Data,BayerStruct);
    else
        Photo(:,:,1)=Data(:,:,1);
        Photo(:,:,2)=(Data(:,:,2)+Data(:,:,3))/2.0;
        Photo(:,:,3)=Data(:,:,4);
    end
    clear Data;
%--------------------------------------------------------------------------
% Get RGB image data
%--------------------------------------------------------------------------
elseif strcmp(Camera.UniqueCameraModel,'PENTAX K-1')==1
    BayerStruct='----';
    BayerStructOrigin='XXXX';
    BlackLevelOrigin=Camera.BlackLevel;
    BayerColorData=CFA;
    n=Camera.BlackLevelRepeatDim(1);
    m=Camera.BlackLevelRepeatDim(2);
    l=length(Camera.BlackLevel)/n/m;
    T=reshape(Camera.BlackLevel,n,m,l);
    for i=1:l
        BlackLevel(:,:,i)=T(:,:,i)';
    end
    WhiteLevelOrigin=Camera.WhiteLevel;
    WhiteLevel=Camera.WhiteLevel;
    CFA=MatWhiteBlackCalibration(CFA,BlackLevel,WhiteLevel);
    PhotoWidth=Camera.DefaultCropSize(1);
    PhotoHeight=Camera.DefaultCropSize(2);
    row=Camera.ActiveArea(1)+Camera.DefaultCropOrigin(2);
    col=Camera.ActiveArea(2)+Camera.DefaultCropOrigin(1);
    Photo=CFA(row+1:row+PhotoHeight,col+1:col+PhotoWidth,:);
elseif strcmp(Camera.UniqueCameraModel,'M1')==1
    BayerStruct='----';
    BayerStructOrigin='XXXX';
    BayerColorData=CFA;
    if isfield(Camera,'BlackLevel')
        BlackLevelOrigin=Camera.BlackLevel;
        BlackLevel=Camera.BlackLevel;
    else
        BlackLevelOrigin='XXXX';
        BlackLevel=0;
    end
    WhiteLevelOrigin=Camera.WhiteLevel;
    WhiteLevel=Camera.WhiteLevel;
    CFA=MatRescaling(CFA,0,WhiteLevel);
    PhotoWidth=Camera.DefaultCropSize(1);
    PhotoHeight=Camera.DefaultCropSize(2);
    row=Camera.DefaultCropOrigin(2);
    col=Camera.DefaultCropOrigin(1);
    Photo=CFA(row+1:row+PhotoHeight,col+1:col+PhotoWidth,:);
else
    disp('Unknown - CFA');
end
save('rawdecode.mat','BayerStruct','BlackLevel','WhiteLevel','-append');
%--------------------------------------------------------------------------
if BayerCfaSave~=0
    disp('-------- Save Bayer CFA Data --------');
    DataCfaFile=strcat(DataPath,strcat(strcat('CFA_',FileName),'.mat'));
    save(DataCfaFile,'BayerColorData','BayerStruct',...
        'BlackLevel','WhiteLevel','UniqueCamera','RawInfo',...
        'BayerStructOrigin','BlackLevelOrigin','WhiteLevelOrigin');
end
%--------------------------------------------------------------------------
end
%--------------------------------------------------------------------------
