%--------------------------------------------------------------------------
function Photo=MatBayerInterpolate(CFA)
%--------------------------------------------------------------------------
load('rawcamera.mat','Camera');
CFA=double(CFA);
if isfield(Camera,'CFARepeatPatternDimension') && ...
        isfield(Camera,'CFAPatternValue')
    if isfield(Camera,'BlackLevelRepeatDim') && ...
            isfield(Camera,'BlackLevel')
        BlackLevel=reshape(Camera.BlackLevel,...
            Camera.BlackLevelRepeatDim(1),...
            Camera.BlackLevelRepeatDim(2))';
    else
        disp('-------- BlackLevel Not Found --------');
        BlackLevel=0;
    end
    WhiteLevel=Camera.WhiteLevel;
    CFA=MatWhiteBlackCalibration(CFA,BlackLevel,WhiteLevel);
    mapstr='rgb';
    CFAPattern=reshape(Camera.CFAPatternValue',...
        Camera.CFARepeatPatternDimension(1),...
        Camera.CFARepeatPatternDimension(2))';
    SensorAlignment=reshape(mapstr(CFAPattern+1)',1,4);
    
    disp('-------- MATLAB Bayer Interpolation --------');
    CFA=uint16(CFA*65535.0);
    Photo=demosaic(CFA,SensorAlignment);
    Photo=double(Photo)/65535.0;
    Photo=max(Photo,0);Photo=min(Photo,1.0);
    
    PhotoHeight=Camera.DefaultCropSize(2);
    PhotoWidth=Camera.DefaultCropSize(1);
    row=Camera.ActiveArea(1)+Camera.DefaultCropOrigin(2);
    col=Camera.ActiveArea(2)+Camera.DefaultCropOrigin(1);
    Photo=Photo(row+1:row+PhotoHeight,col+1:col+PhotoWidth,:);
elseif strcmp(Camera.UniqueCameraModel,'PENTAX K-1')==1
    n=Camera.BlackLevelRepeatDim(1);
    m=Camera.BlackLevelRepeatDim(2);
    l=length(Camera.BlackLevel)/n/m;
    T=reshape(Camera.BlackLevel,n,m,l);
    for i=1:l
        BlackLevel(:,:,i)=T(:,:,i)';
    end
    WhiteLevel=Camera.WhiteLevel;
    CFA=MatWhiteBlackCalibration(CFA,BlackLevel,WhiteLevel);
    PhotoWidth=Camera.DefaultCropSize(1);
    PhotoHeight=Camera.DefaultCropSize(2);
    row=Camera.ActiveArea(1)+Camera.DefaultCropOrigin(2);
    col=Camera.ActiveArea(2)+Camera.DefaultCropOrigin(1);
    Photo=CFA(row+1:row+PhotoHeight,col+1:col+PhotoWidth,:);
elseif strcmp(Camera.UniqueCameraModel,'M1')==1
    BlackLevel=0;
    WhiteLevel=Camera.WhiteLevel;
    CFA=MatRescaling(CFA,BlackLevel,WhiteLevel);
    PhotoWidth=Camera.DefaultCropSize(1);
    PhotoHeight=Camera.DefaultCropSize(2);
    row=Camera.DefaultCropOrigin(2);
    col=Camera.DefaultCropOrigin(1);
    Photo=CFA(row+1:row+PhotoHeight,col+1:col+PhotoWidth,:);
end
save('rawdecode.mat','BlackLevel','WhiteLevel','-append');
%--------------------------------------------------------------------------
end
%--------------------------------------------------------------------------
