%--------------------------------------------------------------------------
% Function:  Matrix of Camera color space to CIE XYZ space 
% Interpolate of Color Matrix & Camera Calibration Matrix
% RawInfo,RawStruct => CameraToXYZ_D50 matrix
%--------------------------------------------------------------------------
function [XyzNeutral,CameraToXYZ]=MatCameraToXYZ(CameraNeutral)
%--------------------------------------------------------------------------
load('rawdecode.mat','RawInfo','AsShotCameraNeutral');
load('rawcamera.mat','Camera');
Iteration=0;

if isfield(Camera,'AnalogBalance')
    AB=[Camera.AnalogBalance(1),0,0;
        0,Camera.AnalogBalance(2),0;
        0,0,Camera.AnalogBalance(3)];
else
    AB=eye(3);
end

if isfield(Camera,'CameraCalibrationSignature') && ...
    isfield(Camera,'ProfileCalibrationSignature') && ...
    strcmp(Camera.CameraCalibrationSignature,...
        Camera.ProfileCalibrationSignature)
    InterpCalibration=1;
else
    InterpCalibration=0;
end

if InterpCalibration==1;
    CC1=reshape(Camera.CameraCalibration1',3,3)';
    CC2=reshape(Camera.CameraCalibration2',3,3)';
    CI1=Camera.CalibrationIlluminant1;
    CI2=Camera.CalibrationIlluminant2;
    CM1=reshape(Camera.ColorMatrix1',3,3)';
    CM2=reshape(Camera.ColorMatrix2',3,3)';
    WeightFactor=[1/2,1/2];
    XYZ=[0.9642;1;0.8249];xyz=GetChromCoord(XYZ);
    for n=1:100
        WeightFactor=GetWeightFactor(CI1,CI2,xyz);
        CM=WeightFactor(1)*CM1+WeightFactor(2)*CM2;
        CC=WeightFactor(1)*CC1+WeightFactor(2)*CC2;
        XYZtoCamera=AB*CC*CM;
        CameraToXYZ=inv(XYZtoCamera);
        XyzNeutral=CameraToXYZ*CameraNeutral;
        xyzNeutral=GetChromCoord(XyzNeutral);
        distance=GetDistance(xyz,xyzNeutral);
        if distance<1.0e-8
            break;
        else
            xyz=xyzNeutral;
        end
    end
    Iteration=n;
else
    WeightFactor=[1,0];
    if isfield(Camera,'CameraCalibration1')
        CC1=reshape(Camera.CameraCalibration1',3,3)';
    else
        disp('-------- CameraCalibration Not Found --------');
        CC1=eye(3);
    end
    CC2=eye(3);
    CC=CC1;
    CI1=Camera.CalibrationIlluminant1;
    CI2=0;
    CI=CI1;
    CM1=reshape(Camera.ColorMatrix1',3,3)';
    CM2=eye(3);
    CM=CM1;
    XYZtoCamera=AB*CC*CM;
    CameraToXYZ=inv(XYZtoCamera);
    XyzNeutral=CameraToXYZ*CameraNeutral;
end

XYZ_White=CameraToXYZ*AsShotCameraNeutral;

StructMatCameraToXYZ=struct(...
    'XyzNeutral',XyzNeutral,'CameraNeutral',CameraNeutral,...
    'AB',AB,'CC',CC,'CC1',CC1,'CC2',CC2,'CM',CM,'CM1',CM1,'CM2',CM2,...
    'CI1',CI1,'CI2',CI2,'Iteration',Iteration,...
    'InterpCalibration',InterpCalibration,'XYZ_White',XYZ_White,...
    'CameraToXYZ',CameraToXYZ,'WeightFactor',WeightFactor);

save('rawdecode.mat','InterpCalibration','WeightFactor',...
    'XyzNeutral','CameraNeutral','StructMatCameraToXYZ','-append');
%--------------------------------------------------------------------------
end
%--------------------------------------------------------------------------
