%--------------------------------------------------------------------------
% Function:  Matrix of Camera color space to CIE XYZ space 
% Interpolate of Color Matrix & Camera Calibration Matrix
% RawInfo,RawStruct => CameraToXYZ_D50 matrix
%--------------------------------------------------------------------------
function CameraToXYZ_D50=MatColorMapping(XyzNeutral)
%--------------------------------------------------------------------------
load('rawdecode.mat','InterpCalibration','WeightFactor',...
    'AsShotCameraNeutral');
load('rawcamera.mat','Camera');

if InterpCalibration~=0
    CI1=Camera.CalibrationIlluminant1;
    CI2=Camera.CalibrationIlluminant2;
    WeightFactor=GetWeightFactor(CI1,CI2,GetChromCoord(XyzNeutral));
    save('rawcamera.mat','WeightFactor','-append');
end

if isfield(Camera,'AnalogBalance')
    AB=[Camera.AnalogBalance(1),0,0;
        0,Camera.AnalogBalance(2),0;
        0,0,Camera.AnalogBalance(3)];
else
    AB=eye(3);
end

if isfield(Camera,'ForwardMatrix1') && isfield(Camera,'ForwardMatrix2')
    ForwardMatrix=1;
    FM1=reshape(Camera.ForwardMatrix1',3,3)';
    FM2=reshape(Camera.ForwardMatrix2',3,3)';
elseif isfield(Camera,'ForwardMatrix1')
    ForwardMatrix=1;
    disp('--------- Only one ForwardMatrix found ----------');
    FM1=reshape(Camera.ForwardMatrix1',3,3)';
    FM2=eye(3);
    save('rawdecode.mat','FM1','-append');
    INTERRUPT
else
    ForwardMatrix=0;
    FM1=eye(3);
    FM2=eye(3);
end

if InterpCalibration==1
    CC1=reshape(Camera.CameraCalibration1',3,3)';
    CC2=reshape(Camera.CameraCalibration2',3,3)';
    CC=WeightFactor(1)*CC1+WeightFactor(2)*CC2;
    CM1=reshape(Camera.ColorMatrix1',3,3)';
    CM2=reshape(Camera.ColorMatrix2',3,3)';
    CM=WeightFactor(1)*CM1+WeightFactor(2)*CM2;
    FM=WeightFactor(1)*FM1+WeightFactor(2)*FM2;
else
    if isfield(Camera,'CameraCalibration1')
        CC1=reshape(Camera.CameraCalibration1',3,3)';
    else
        disp('-------- CameraCalibration Not Found --------');
        CC1=eye(3);
    end
    CC2=eye(3);
    CC=CC1;
    CM1=reshape(Camera.ColorMatrix1',3,3)';
    CM2=eye(3);
    CM=CM1;
    FM=FM1;
end

% Test 80D red color 
% ForwardMatrix=0;

if ForwardMatrix==1
    XYZtoCamera=AB*CC*CM;
    CameraNeutral=XYZtoCamera*XyzNeutral;
    ReferenceNeutral=inv(AB*CC)*CameraNeutral;
    D=inv([ ReferenceNeutral(1),0,0;...
        0,ReferenceNeutral(2),0;...
        0,0,ReferenceNeutral(3)]);
    CameraToXYZ_D50=FM*D*inv(AB*CC);
    CA=eye(3);
else
    D50=[0.9642;1;0.8249];
    XYZtoCamera=AB*CC*CM;
    CameraNeutral=XYZtoCamera*XyzNeutral;
    CameraToXYZ=inv(XYZtoCamera);
    CA=MatBradfordAlgorithm(XyzNeutral,D50);
    CameraToXYZ_D50=CA*CameraToXYZ;
end    

XYZ_White=CameraToXYZ_D50*AsShotCameraNeutral;

StructMatColorMapping=struct(...
    'XyzNeutral',XyzNeutral,'CameraNeutral',CameraNeutral,...
    'AB',AB,'CC',CC,'CC1',CC1,'CC2',CC2,'CM',CM,'CM1',CM1,'CM2',CM2,...
    'FM',FM,'FM1',FM1,'FM2',FM2,'CA',CA,...
    'ForwardMatrix',ForwardMatrix,'XYZ_White',XYZ_White,...
    'CameraToXYZ_D50',CameraToXYZ_D50,'WeightFactor',WeightFactor);
save('rawdecode.mat','ForwardMatrix','XyzNeutral','CameraNeutral',...
    'StructMatColorMapping','-append');
%--------------------------------------------------------------------------
end
%--------------------------------------------------------------------------
