%--------------------------------------------------------------------------
% Raw Decode processing
%--------------------------------------------------------------------------
function RawDecode(FileName,Exposure)
%--------------------------------------------------------------------------
copyright='rivershaw';
disp('++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++');
disp(FileName);
%----------------- Initial data file --------------------------------------
disp('Initialize');
load('rawconfig.mat','DngPath','TiffPath','ColorSpace','WorkColorSpace',...
    'AutoWhiteBalance','Constrast','Saturation',...
    'UseProfileToneCurve','UseLrToneCurve',...
    'Histogram','HistCentre','HistRange','TiffShow','DecodeOutput');
save('rawdecode.mat','copyright','FileName','Exposure',...
    'UseProfileToneCurve','UseLrToneCurve');
save('rawdebug.mat','FileName');
FigSequence=1;
save('rawconfig.mat','FigSequence','-append');
ApexRgb=[1,0,0,1;0,1,0,0;0,0,1,0];
ApexXyz=[1,0,0,1;0,1,0,0;0,0,1,0];
Apex.val=ApexXyz;Apex.color=[1,0,0];Apex.point='o';Apex.style=0;
save('rawapex.mat','ApexRgb','ApexXyz','Apex');
disp('Done!');
%--------------------------------------------------------------------------

%----------------- Read DNG image file data -------------------------------
disp('Read Raw(dng) data');
load('rawconfig.mat','DngPath','TiffPath');
DngFilePathName=strcat(DngPath,FileName);
TiffFilePathName=strcat(TiffPath,FileName);
[cfa,CameraNeutral]=RawReadImageData(DngFilePathName);
disp('Done!');
%--------------------------------------------------------------------------

%----------------- Bayer Extraction ---------------------------------------
disp('Bayer Extraction');
PhotoRaw=RawBayerExtraction(cfa);
clear cfa;
disp('Done!');
%--------------------------------------------------------------------------

%----------------- Expansion ----------------------------------------------
disp('Expansion');
ImageRgbRaw=RawExpansion(PhotoRaw,Exposure);
disp('Done!');
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
if DecodeOutput~=0
    %----------------- Camera Color Space to XYZ --------------------------
    disp('Camera Color Space to XYZ');
    [XyzNeutral,ImageXyzRaw]=RawSpaceCameraToXyz(ImageRgbRaw,...
        CameraNeutral);
    disp('Done!');
    %----------------------------------------------------------------------
    
    %------------------- Auto White Balance -------------------------------
    if AutoWhiteBalance~=0
        disp('Auto White Balance');
        XyzNeutral=GetNeutralGrayReflect(ImageXyzRaw);
        disp('Done!');
    end
    %----------------------------------------------------------------------
    
    %----------- Camera Color Space to XYZ_D50 ----------------------------
    disp('Camera Color Space to XYZ_D50');
    ImageXyz=RawSpaceCameraToXyz_D50(ImageRgbRaw,XyzNeutral);
    disp('Done!');
    %----------------------------------------------------------------------
    
    %----------------- Profile HSV Calibration ----------------------------
    disp('Profile HSV Calibration');
    ImageXyz=RawProfileHsvCalibration(ImageXyz);
    disp('Done!');
    %----------------------------------------------------------------------
    
    %--------- XYZ_D50 to sRGB, Adobe RGB 1998 or ProPhoto RGB-------------
    disp(strcat(strcat('XYZ_D50 to [',WorkColorSpace),']'));
    ImageRgb=RawSpaceXyz2Rgb(ImageXyz,WorkColorSpace);
    disp('Done!');
    %----------------------------------------------------------------------
    
    %------------------- Exposure Calibration -----------------------------
    disp('Exposure Calibration');
    ImageRgb=RawExposureCalibration(ImageRgb);
%     save('rawdebug.mat','ImageRgb','-append')
    disp('Done!');
    %----------------------------------------------------------------------
    
    %------------------ Constrast -----------------------------------------
    if Constrast~=1
        disp(strcat('Constrast:',num2str(Constrast,'%+5.2f')));
        ImageRgb=RawConstrast(ImageRgb,Constrast,WorkColorSpace);
        disp('Done!');
    end
    %----------------------------------------------------------------------
    
    %------------------ Saturation ----------------------------------------
    if Saturation~=1
        disp(strcat('Saturation:',num2str(Saturation,'%+5.2f')));
        ImageRgb=RawSaturation(ImageRgb,Saturation);
        disp('Done!');
    end
    %----------------------------------------------------------------------
    
    %------------------- Applied Profile Look Table -----------------------
    disp('Applied Profile Look Table');
    ImageRgb=RawProfileLookTable(ImageRgb);
    disp('Done!');
    %----------------------------------------------------------------------

    %------------------- Profile Tone Curve -------------------------------
    if UseProfileToneCurve~=0
        disp('Profile Tone Curve');
        ImageRgb=RawProfileToneCurve(ImageRgb);
        disp('Done!');
    end
    %----------------------------------------------------------------------
    
    %------------------- Lightroom Tone Curve -----------------------------
    if UseLrToneCurve~=0
        disp('Lightroom Tone Curve');
        ImageRgb=RawLrToneCurve(ImageRgb);
        disp('Done!');
    end
    %----------------------------------------------------------------------
    
    %------------------ Gamma Correction ----------------------------------
    disp('Gamma Correction');
    if strcmp(ColorSpace,WorkColorSpace)
        ImageRgb=RawGammaCorrect(ImageRgb,WorkColorSpace);
    else
        ImageXyz=RawSpaceRgb2Xyz(ImageRgb,WorkColorSpace);
        ImageRgb=RawSpaceXyz2Rgb(ImageXyz,ColorSpace);
        ImageRgb=RawGammaCorrect(ImageRgb,ColorSpace);
    end
    disp('Done!');
    %----------------------------------------------------------------------
    
    %------------------ Save Tiff Photograph ------------------------------
    disp('Save Tiff Photograph');
    RawTiffPhoto(ImageRgb,TiffFilePathName);
    disp('Done!');
    %----------------------------------------------------------------------
    
    %------------------ Data save by file name ----------------------------
    RawDataDebug();
    %----------------------------------------------------------------------
end
%--------------------------------------------------------------------------
end
%--------------------------------------------------------------------------
